"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.raw = void 0;
const file_loader_1 = __importDefault(require("file-loader"));
const mime_1 = __importDefault(require("mime"));
const path_1 = __importDefault(require("path"));
const replace_ext_1 = __importDefault(require("replace-ext"));
const schema_utils_1 = require("schema-utils");
const sharp_1 = __importDefault(require("sharp"));
const webpack_loader_util_1 = require("@calvin-l/webpack-loader-util");
const options_json_1 = __importDefault(require("./options.json"));
exports.raw = true;
function default_1(content, sourceMap) {
    const callback = this.async();
    const defaultOptions = {
        format: getFormat(this.resourcePath),
        scaleUp: false,
    };
    const options = {
        ...defaultOptions,
        ...webpack_loader_util_1.getOptions(this, true, true),
    };
    schema_utils_1.validate(options_json_1.default, options, {
        name: "Image Resize Loader",
        baseDataPath: "options",
    });
    processImage(content, options)
        .then((result) => {
        const fileLoaderContext = {
            ...this,
            resourcePath: replaceExtension(this.resourcePath, options.format),
            query: options.fileLoaderOptions,
        };
        const fileLoaderResult = file_loader_1.default.call(fileLoaderContext, result, sourceMap);
        callback(null, fileLoaderResult);
    })
        .catch((e) => {
        throw e;
    });
}
exports.default = default_1;
async function processImage(content, { width, height, fit, position, background, scale, format, quality, scaleUp, sharpOptions, }) {
    let sharpImage = sharp_1.default(Buffer.from(content));
    const { height: imageHeight, width: imageWidth, } = await sharpImage.metadata();
    const normalizedImageHeight = imageHeight !== null && imageHeight !== void 0 ? imageHeight : Number.MAX_VALUE;
    const normalizedImageWidth = imageWidth !== null && imageWidth !== void 0 ? imageWidth : Number.MAX_VALUE;
    const normalizedResultHeight = height !== null && height !== void 0 ? height : 0;
    const normalizedResultWidth = width !== null && width !== void 0 ? width : 0;
    if ((width || height) &&
        (scaleUp ||
            (normalizedResultHeight <= normalizedImageHeight &&
                normalizedResultWidth <= normalizedImageWidth)))
        sharpImage = sharpImage.resize({
            width,
            height,
            fit,
            position,
            background,
            ...sharpOptions === null || sharpOptions === void 0 ? void 0 : sharpOptions.resize,
        });
    else if (scale && (scaleUp || scale <= 1)) {
        sharpImage = sharpImage.resize({
            width: Math.round(normalizedImageWidth * scale),
            fit,
            position,
            background,
            ...sharpOptions === null || sharpOptions === void 0 ? void 0 : sharpOptions.resize,
        });
    }
    if (format)
        sharpImage = sharpImage[format]({ quality, ...sharpOptions === null || sharpOptions === void 0 ? void 0 : sharpOptions[format] });
    return await sharpImage.toBuffer();
}
function replaceExtension(resourcePath, format) {
    if (!format)
        return resourcePath;
    if (mime_1.default.getType(format) === mime_1.default.getType(path_1.default.extname(resourcePath)))
        return resourcePath;
    return replace_ext_1.default(resourcePath, `.${format}`);
}
function getFormat(resourcePath) {
    const type = mime_1.default.getType(resourcePath);
    switch (type) {
        case "image/jpeg":
            return "jpeg";
        case "image/png":
            return "png";
        case "image/webp":
            return "webp";
        case "image/tiff":
            return "tiff";
    }
}
//# sourceMappingURL=index.js.map